%=============================================================================
% Copyright (c) 2016-present Allan CORNET (Nelson)
%=============================================================================
% This file is part of the Nelson.
%=============================================================================
% LICENCE_BLOCK_BEGIN
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU Lesser General Public
% License as published by the Free Software Foundation; either
% version 2.1 of the License, or (at your option) any later version.
%
% Alternatively, you can redistribute it and/or
% modify it under the terms of the GNU General Public License as
% published by the Free Software Foundation; either version 2 of
% the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU Lesser General Public License for more details.
%
% You should have received a copy of the GNU Lesser General Public
% License along with this program. If not, see <http://www.gnu.org/licenses/>.
% LICENCE_BLOCK_END
%=============================================================================
function ce = cellstr(obj)
%## Converts to cell of character array.
%### Syntax
%    ce = cellstr(A)
%### Input argument
%A
%
%       a string, a string array, cell of character array.
%
%### Output argument
%ce
%
%       a cell of character array.
%### Description
%<b>cellstr(A)</b> converts to cell of character array.
%### Example
%<pre>
%cellstr('Nelson')
%cellstr({'Nelson'})
%cellstr({})
%</pre>

  if iscellstr(obj)
    ce = obj;
  elseif isstring(obj)
    if ~any(ismissing(obj))
      ce = cell(obj);
    else
      error(_('Cannot convert missing element.'));
    end
  elseif iscell(obj)
    error(_('Cell must be string scalars or character arrays.'));
  elseif ischar(obj)
    if isempty(obj)
      ce = {''};
    elseif ndims(size(obj)) != 2
      error(_('Matrix 2D expected.'));
    else
      numrows = size(obj,1);
      ce = cell(numrows,1);
      for i = 1:numrows
        ce{i} = obj(i, :);
      end
      ce = deblank(ce);
    end
  else
    error(_(['Type not supported:' ,' ', class(obj)]));
  end
end
%=============================================================================
